
clear
set more off
set matsize 5000


// create or overwrite output file in ascii format

capture log close
log using logfiles/bma_datamaker.log, replace                       


************************************************************************************************************
//
//		THIS PROGRAM BUILDS THE DATA SET FOR THE BAYESIAN MODEL AVERAGING ANALYSIS OF THE 1979-2002 TREND FIELD
//
//		COMPONENT A TAKES THE MM07 DATA SET, RETAINING ALL 469 ROWS
//
//		COMPONENT B BUILDS THE OSCILLATION DATA AND MERGES WITH THE COMPONENT A DATA, THEN REDUCES TO 440 ROWS
//
//		COMPONENT C TAKES THE A&B DATA AND MERGES WITH THE RSS, CRU AND GISS DATA + CHAD'S GCM DATA
//
//		COMPONENT D GENERATES SUMMARY STATS AND WRITES THE DATA BASE
//
//		COMPONENT E GENERATES SPATIAL LAG WEIGHTS
//
************************************************************************************************************

************************************************************************************************************
//				START OF COMPONENT A
************************************************************************************************************

//															COMPONENT A
//           Clear data then reads data from MMJGR07.csv 
clear
set memory 50m
insheet using data/MMJGR07.csv                                

//  cru = surface trend as used in MM07; cru2v, cru3v provided by G Schmidt

//															COMPONENT A

generate abslat=abs(lat)
* replace abslat = cos(lat*3.1415927/180)  //       results rerun using coslat yields only minor differences
generate coslat = cos(lat*3.1415927/180)
generate press=slp
generate dry = dslp>0

//															COMPONENT A

//              SURFACE PROCESSES: % growth population, income, GDP & Coal use
//              land is in sq km, pop is in millions; scale popden to persons/km2
//              gdp is in trillions; gdpden is in $millions/km2 

generate p79 =  1000000*pop79/land         
generate p99 =  1000000*pop99/land         


generate g79 =  1000000*gdp79/land
generate g99 =  1000000*gdp99/land 
generate g=g79
//															COMPONENT A


//            Growth in % terms

generate poscoal80 = coal80>0                                              // dummy for coal use in 1980 >0 to avoid div0 error
generate c = poscoal80*(coal00/(coal80+(1-poscoal80)*.001)) - poscoal80    // % change from 1980 to 2000

sum(c coal00 coal80)                                                       // if coal80 = 0 then c == 0, but for these countries the 
sum(c coal00 coal80) if poscoal==0                                         // change in coal use was very small

generate y = gdp99/gdp79 - 1                                               // % change from 1979 to 1999

generate m = inc99/inc79 - 1                                                // % change from 1979 to 1999

generate p = pop99/pop79 - 1                                                // % change from 1979 to 1999


//															COMPONENT A


//             DATA INHOMOGENEITIES: Resources g79 (1979 GDP/area ), educ (literacy + PSE)
generate e = lit+pse                                                        // educ = sum of Lit and PSE
generate ge=g79*e                                                           // instruments for Hausman Test
generate g2=g*g
generate e2=e*e
generate x=missing

//              Count cells with lost records 
sum(missing) if missing>12
sum(missing) if missing==0
//															COMPONENT A


keep lat lon cru uah slp dry dslp water abslat coslat pse gdp99 g e x p m y c 

save data/t_econ_469, replace


//															COMPONENT A
************************************************************************************************************
//						END OF COMPONENT A
************************************************************************************************************









************************************************************************************************************
//						START OF COMPONENT B
************************************************************************************************************



***********************************************************************************
**************************   Module for AO regression field file  **************
***********************************************************************************

/* lat-lon data */
clear
insheet using data/latlon.csv
//															COMPONENT B

/* position markers: row, column, corr coeff (matrix element) */
ge corcoef=0
ge double r=(90-lat)/2.5+1
ge double c=(180+lon)/2.5+1
keep lat lon r c corcoef
//															COMPONENT B

/*  Merge corr coeff matrix and lat-lon records  */
/*  Replace insheet with NOAA data file          */
save data/rcc ,replace
clear
insheet cm1-cm144 using data/AO.reg.csv 
save data/cm ,replace
clear
use data/rcc
merge using data/cm
//															COMPONENT B

/*  position scalars  */
scalar def rs=0
scalar def cs=0

/* make regression coefficient matrix of NOAA data columns and final vector*/
mkmat cm*, mat(ccm)
mkmat corcoef
//															COMPONENT B

/*  extract regression coef and place in data vector */
forv z = 1/469 {
               scalar def rs=r[`z']
               scalar def cs=c[`z']
             *   scalar list rs cs  
               matrix corcoef[`z',1] = ccm[rs,cs]
             ** display lat[`z'] "   " lon[`z'] "   " corcoef[`z',1]
               }
//															COMPONENT B

/*  convert corcoef matrix to variable  and drop everything else */
/*  This name must match NOAA matrix origin  */
svmat double corcoef, name(aorv)
rename aorv1 aorv
keep aorv
save data/aorv, replace

/*      End of module  */
//															COMPONENT B


***********************************************************************************
**************************   Module for NAO regression field file  **************
***********************************************************************************

/* lat-lon data */
clear
insheet using data/latlon.csv
//															COMPONENT B

/* position markers: row, column, corr coeff (matrix element) */
ge corcoef=0
ge double r=(90-lat)/2.5+1
ge double c=(180+lon)/2.5+1
keep lat lon r c corcoef

/*  Merge corr coeff matrix and lat-lon records  */
/*  Replace insheet with NOAA data file          */
save data/rcc ,replace
clear
insheet cm1-cm144 using data/NAO.reg.csv 
save data/cm ,replace
clear
use data/rcc
merge using data/cm
//															COMPONENT B

/*  position scalars  */
scalar def rs=0
scalar def cs=0

/* make regression coefficient matrix of NOAA data columns and final vector*/
mkmat cm*, mat(ccm)
mkmat corcoef
//															COMPONENT B

/*  extract regression coef and place in data vector */
forv z = 1/469 {
               scalar def rs=r[`z']
               scalar def cs=c[`z']
             *   scalar list rs cs  
               matrix corcoef[`z',1] = ccm[rs,cs]
             ** display lat[`z'] "   " lon[`z'] "   " corcoef[`z',1]
               }

/*  convert corcoef matrix to variable  and drop everything else */
/*  This name must match NOAA matrix origin  */
svmat double corcoef, name(naorv)
rename naorv1 naorv
keep naorv
save data/naorv, replace

/*      End of module  */
//															COMPONENT B



***********************************************************************************
**************************   Module for PDO regression field file  **************
***********************************************************************************

/* lat-lon data */
clear
insheet using data/latlon.csv
//															COMPONENT B

/* position markers: row, column, corr coeff (matrix element) */
ge corcoef=0
ge double r=(90-lat)/2.5+1
ge double c=(180+lon)/2.5+1
keep lat lon r c corcoef
//															COMPONENT B

/*  Merge corr coeff matrix and lat-lon records  */
/*  Replace insheet with NOAA data file          */
save data/rcc ,replace
clear
insheet cm1-cm144 using data/PDO.reg.csv 
save data/cm ,replace
clear
use data/rcc
merge using data/cm
//															COMPONENT B

/*  position scalars  */
scalar def rs=0
scalar def cs=0

/* make regression coefficient matrix of NOAA data columns and final vector*/
mkmat cm*, mat(ccm)
mkmat corcoef
//															COMPONENT B

/*  extract regression coef and place in data vector */
forv z = 1/469 {
               scalar def rs=r[`z']
               scalar def cs=c[`z']
             *   scalar list rs cs  
               matrix corcoef[`z',1] = ccm[rs,cs]
             * display lat[`z'] "   " lon[`z'] "   " corcoef[`z',1]
               }
//															COMPONENT B

/*  convert corcoef matrix to variable  and drop everything else */
/*  This name must match NOAA matrix origin  */
svmat double corcoef, name(pdorv)
rename pdorv1 pdorv
keep pdorv
save data/pdorv, replace

/*      End of module  */
//															COMPONENT B


***********************************************************************************
**************************   Module for SOI regression field file  **************
***********************************************************************************

/* lat-lon data */
clear
insheet using data/latlon.csv
//															COMPONENT B

/* position markers: row, column, corr coeff (matrix element) */
ge corcoef=0
ge double r=(90-lat)/2.5+1
ge double c=(180+lon)/2.5+1
keep lat lon r c corcoef
//															COMPONENT B

/*  Merge corr coeff matrix and lat-lon records  */
/*  Replace insheet with NOAA data file          */
save data/rcc ,replace
clear
insheet cm1-cm144 using data/SOI.reg.csv 
save data/cm ,replace
clear
use data/rcc
merge using data/cm
//															COMPONENT B

/*  position scalars  */
scalar def rs=0
scalar def cs=0

/* make regression coefficient matrix of NOAA data columns and final vector*/
mkmat cm*, mat(ccm)
mkmat corcoef
//															COMPONENT B

/*  extract regression coef and place in data vector */
forv z = 1/469 {
               scalar def rs=r[`z']
               scalar def cs=c[`z']
             *   scalar list rs cs  
               matrix corcoef[`z',1] = ccm[rs,cs]
             * display lat[`z'] "   " lon[`z'] "   " corcoef[`z',1]
               }
//															COMPONENT B

/*  convert corcoef matrix to variable  and drop everything else */
/*  This name must match NOAA matrix origin  */
svmat double corcoef, name(soirv)
rename soirv1 soirv
keep soirv
save data/soirv, replace

/*      End of module  */

//															COMPONENT B


*******************************************************************************
//
//                     Build data set for regressions
//
*******************************************************************************
clear

use data/t_econ_469             // call complete dataset "cdata"
save data/cdata, replace

//															COMPONENT B

use data/aorv
merge using data/cdata
drop _merge
    save data/cdata, replace
    clear
//															COMPONENT B

use data/naorv
merge using data/cdata
drop _merge
    save data/cdata, replace
    clear
//															COMPONENT B

use data/pdorv
merge using data/cdata
drop _merge
    save data/cdata, replace
    clear
//															COMPONENT B

use data/soirv
merge using data/cdata
drop _merge
    save data/cdata, replace
    clear


use data/cdata
//           DROP ANTARCTIC DATA	& ROWS WITH MISSING CRU TRENDS. THIS LEAVES 440 ROWS			
	drop if lat < -67
	drop if cru>200    
	sum               
	save data/cdata, replace
clear

************************************************************************************************************
//				END OF COMPONENT B
************************************************************************************************************





*****************************************************************************************************
//				START OF COMPONENT C
//
//		TAKE THE COMPONENT A&B DATA, MERGE WITH THE RSS, CRU AND GISS DATA + CHAD'S GCM DATA  
// 		UPDATE OCTOBER 09: Gavin issued a corrigendum with new data, which is in the giss2.csv spreadsheet
*****************************************************************************************************


insheet using data/giss2.csv           		// corrected GISS data 
	mvdecode _all, mv(-9999)                  // converts CRU and RSS data -9999 to Missing value code
	drop giss?t giss?s latgs longs
	save data/gs, replace
//															COMPONENT C


use data/cdata                                 
	merge using data/gs
	drop _merge
	save data/mg, replace
	clear
//															COMPONENT C


// 	Read in CHAD's GCM DATA
insheet using data/trend.fields.chad.noname.csv  	// Chad's GCM data
	merge using data/mg
	describe

//                                         convert from deg/year to deg/decade
replace cru2v=cru2v*10
replace cru3v=cru3v*10
replace rss=rss*10

//															COMPONENT C


//			Identify Latitude Bands
generate us = pse>45
generate nh = lat>0
generate sh = lat<0
generate tr = abslat<30

//              Generate country dummy variables using gdp99 (identical within a country) for clustering

quietly tabulate gdp99, generate(gdp99)                   

//															COMPONENT C

*		GROUP INTO ENSEMBLE MEANS AND x10 TO GET DEG/YEAR
 generate ms1=10*(msurf1)
 generate ms2=10*(msurf2+msurf3+msurf4+msurf5+msurf6)/5
 generate ms3=10*(msurf7)
 generate ms4=10*(msurf8)
 generate ms5=10*(msurf9)
 generate ms6=10*(msurf10)
 generate ms7=10*(msurf11)
 generate ms8=10*(msurf12+msurf13)/2
 generate ms9=10*(msurf14+msurf15+msurf16+msurf17+msurf18)/5
 generate ms10=10*(msurf19+msurf20+msurf21+msurf22+msurf23)/5
 generate ms11=10*(msurf24+msurf25+msurf26)/3
 generate ms12=10*(msurf27)
 generate ms13=10*(msurf28)
 generate ms14=10*(msurf29)
 generate ms15=10*(msurf30)
 generate ms16=10*(msurf31+msurf32+msurf33)/3
 generate ms17=10*(msurf34+msurf35+msurf36+msurf37+msurf38)/5
 generate ms18=10*(msurf39+msurf40+msurf41+msurf42)/4
 generate ms19=10*(msurf43+msurf44+msurf45+msurf46+msurf47+msurf48+msurf49)/7
 generate ms20=10*(msurf50+msurf51+msurf52+msurf53)/4
 generate ms21=10*(msurf54)
 generate ms22=10*(msurf55)


 generate mt1=10*(mtrop1)
 generate mt2=10*(mtrop2+mtrop3+mtrop4+mtrop5+mtrop6)/5
 generate mt3=10*(mtrop7)
 generate mt4=10*(mtrop8)
 generate mt5=10*(mtrop9)
 generate mt6=10*(mtrop10)
 generate mt7=10*(mtrop11)
 generate mt8=10*(mtrop12+mtrop13)/2
 generate mt9=10*(mtrop14+mtrop15+mtrop16+mtrop17+mtrop18)/5
 generate mt10=10*(mtrop19+mtrop20+mtrop21+mtrop22+mtrop23)/5
 generate mt11=10*(mtrop24+mtrop25+mtrop26)/3
 generate mt12=10*(mtrop27)
 generate mt13=10*(mtrop28)
 generate mt14=10*(mtrop29)
 generate mt15=10*(mtrop30)
 generate mt16=10*(mtrop31+mtrop32+mtrop33)/3
 generate mt17=10*(mtrop34+mtrop35+mtrop36+mtrop37+mtrop38)/5
 generate mt18=10*(mtrop39+mtrop40+mtrop41+mtrop42)/4
 generate mt19=10*(mtrop43+mtrop44+mtrop45+mtrop46+mtrop47+mtrop48+mtrop49)/7
 generate mt20=10*(mtrop50+mtrop51+mtrop52+mtrop53)/4
 generate mt21=10*(mtrop54)
 generate mt22=10*(mtrop55)

//															COMPONENT C

keep lat lon cru cru3v uah4 rss4 slp dry dslp water abslat aorv naorv pdorv soirv g e x p m y c ms1-ms22 mt1-mt22

sum

save data/cdata, replace



************************************************************************************************************
//				END OF COMPONENT C
************************************************************************************************************




*****************************************************************************************************
//				START OF COMPONENT D
//
//				SUMMARY STATS AND WRITE TO OUTPUT FILE
*****************************************************************************************************


//	
use data/cdata

generate msu=(uah4+rss4)/2
drop if cru3v==.
drop if msu==.

outsheet lat lon cru3v msu aorv naorv pdorv soirv slp dry dslp water abslat g e x p m y c ///
	ms1-ms22 using data/climate_bma.csv, replace c

summarize


*****************************************************************************************************
//				START OF COMPONENT E
//
//				GENERATE SPATIAL LAG WEIGHTS: CRU3v version
*****************************************************************************************************

	
/* convert latitude and longitude in radians */
	ge rlat = (lat*_pi)/180
	ge rlon = (lon*_pi)/180
//															*	COMPONENT D 	*

ge ord = _n

//															*	COMPONENT D 	*

glo size = _N
forv c = 1/$size		{

	  qui generate varW`c' = 0
	  qui forv r = 1/$size 	{

	    * First determine distances (using km)
 	   replace varW`c' = 6371 * (acos(sin(rlat[`c'])*sin(rlat[`r'])+ ///
		cos(rlat[`c'])*cos(rlat[`r'])*cos(rlon[`r']-rlon[`c']))) in `r' if `r' ~= `c'

	    * Any weighting scheme (where Wrc is a function of Drc) can then be introduced here.
	    replace varW`c' = 1/((varW`c')^2.3) in `r' if `r' ~= `c'
						}

				}

//															*	COMPONENT D 	*



/* Save information */
save data/tmp, replace
	keep varW*
	save data/Wcru3.dta, replace
	clear
	use data/tmp


exit


